package com.sitemesh.taglib.page;

import com.sitemesh.*;
import com.sitemesh.filter.PageResponse;
import com.sitemesh.taglib.AbstractTag;
import java.io.IOException;
import java.net.URL;
import javax.servlet.Config;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * This tag inserts an external resource as a panel into the current Page.
 *
 * <p>The page attribute is required and points to the panel resource
 * which should expose an entire page (e.g. another JSP file producing
 * HTML). This attribute can be relative to the page it is being called
 * from or an absolute path from the context-root.</p>
 *
 * <p>The (optional) decorator attribute is the name of the Decorator
 * to apply to the included page. Note that the implementation of
 * DecoratorMapper can overide this.</p>
 *
 * @author <a href="mailto:joe@truemesh.com">Joe Walnes</a>
 * @version $Revision: 1.5 $
 */
public class PanelTag extends AbstractTag {

	private String page;
	private String decorator;

	/**
	 * Tag attribute: URI of page to include.
	 * Can be relative to page being called from, or absolute
	 * path from context-root of web-app.
	 */
	public void setPage( String page ) {
		this.page = page;
	}

	/**
	 * Tag attribute: Name of Decorator to apply to Page.
	 * This is passed to DecoratorMapper to retrieve appropriate
	 * Decorator. DecoratorMapper may override if needed.
	 *
	 * @see com.sitemesh.DecoratorMapper
	 */
	public void setDecorator( String decorator ) {
		this.decorator = decorator;
	}

	/**
	 * Standard taglib method: resets attributes.
	 */
	public void release() {
		setPage( null );
		setDecorator( null );
		super.release();
	}

	/**
	 * Standard taglib method: apply decorator to page.
	 */
	public int doEndTag() throws JspException {
		try {
			Page oldPage = (Page)pageContext.getRequest().getAttribute(PAGE);
			Page pageObj = getPage(page);
			pageContext.getRequest().setAttribute(DECORATOR, decorator);
			pageContext.getRequest().setAttribute(PAGE, pageObj);
			Decorator d = getPanelDecorator(pageObj);
			if ( d != null ) {
				pageContext.include( d.getPage() );
			}
			else {
				// decorator not found
				// @todo: error handling
			}
			pageContext.getRequest().setAttribute(PAGE, oldPage);
			return EVAL_PAGE;
		}
		catch (IOException e) {
			throw new JspException(e.toString());
		}
		catch (ServletException e) {
			throw new JspException(e.toString());
		}
	}

	/**
	 * Obtain Page object for page specified in {@link #setPage(java.lang.String)} .
	 */
	private Page getPage( String page ) throws IOException, ServletException {
		// @todo: remove "text/html" hard-code.
		String contentType        = "text/html";
		Config config             = pageContext.getServletConfig();
		PageParser parser         = Factory.getInstance(config).getPageParser( contentType );

		PageResponse pageResponse = new PageResponse( ( HttpServletResponse )pageContext.getResponse(), config );
		RequestDispatcher rd      = pageContext.getRequest().getRequestDispatcher( page );
		rd.forward( pageContext.getRequest(), pageResponse );

		Page result = pageResponse.getPage();
		result.setRequest( (HttpServletRequest) pageContext.getRequest() );
		return result;
	}

	/**
	 * Obtain Decorator object for page specified in {@link #setDecorator(java.lang.String)} .
	 */
	private Decorator getPanelDecorator( Page page ) throws ServletException {
		DecoratorMapper decoratorMapper = Factory.getInstance(pageContext.getServletConfig()).getDecoratorMapper();
		return decoratorMapper.getDecorator( ((HttpServletRequest)pageContext.getRequest()), page );
	}

}
